import java.net.URI;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;

import com.kenai.jbosh.BOSHClient;
import com.kenai.jbosh.BOSHClientConfig;
import com.kenai.jbosh.BOSHClientConnEvent;
import com.kenai.jbosh.BOSHClientConnListener;
import com.kenai.jbosh.BOSHClientRequestListener;
import com.kenai.jbosh.BOSHClientResponseListener;
import com.kenai.jbosh.BOSHMessageEvent;
import com.kenai.jbosh.ComposableBody;


public class BoshTest {

	private static final String URL = "http://localhost:7070/http-bind/";
	private static final String XMPP_DOMAIN = "localhost";

	static private boolean finished = false;

	public static void main(String[] args) {
		try {
			// Parsing CMD Args
			URI uri;
			String xmppDomain;
			Options options = new Options();
			options.addOption(
					"u",
					true,
					"The URL of the BOSH service (default: '" + URL + "').");
			options.addOption(
					"x",
					true,
					"The XMPP domain of the connection (default: '" + XMPP_DOMAIN + "').");
			options.addOption("h", false, "Print this help.");
			CommandLineParser parser = new PosixParser();
			CommandLine cmd = parser.parse(options, args);
			if (cmd.hasOption('u')) {
				uri = new URI(cmd.getOptionValue('u'));
			}
			else {
				uri = new URI(URL);
			}
			if (cmd.hasOption('x')) {
				xmppDomain = cmd.getOptionValue('x');
			}
			else {
				xmppDomain = XMPP_DOMAIN;
			}
			if (cmd.hasOption('h')) {
				HelpFormatter formatter = new HelpFormatter();
				formatter.printHelp("bosh-test", options);
				return;
			}
			
			// Init BOSH client
			BOSHClientConfig cfg = BOSHClientConfig.Builder.create(
					uri, xmppDomain)
					.build();
			BOSHClient client = BOSHClient.create(cfg);
			
			client.addBOSHClientConnListener(new BOSHClientConnListener () {
					public void connectionEvent(BOSHClientConnEvent connEvent) {
						if (connEvent.isConnected()) {
							System.out.println("Successfully connected to " + URL);
						}
						else {
							System.err.println("Not connected to " + URL);
						}
						finished = true;
					}
				});
			client.addBOSHClientResponseListener(new BOSHClientResponseListener () {
					public void responseReceived(BOSHMessageEvent event) {
						if (event.getBody() != null) {
							System.out.println("RECV: " + event.getBody().toXML());
						}
					}
				});
			client.addBOSHClientRequestListener(new BOSHClientRequestListener () {
					public void requestSent(BOSHMessageEvent event) {
						if (event.getBody() != null) {
							System.out.println("SENT: " + event.getBody().toXML());
						}
					}
				});
			
			client.send(
					ComposableBody.builder()
						.setNamespaceDefinition("xmpp", "urn:xmpp:xbosh")
						.build());
			
			while (!finished) {
				Thread.sleep(60);
			}
		}
		catch (Exception e) {
			System.err.print(e.getMessage());
			e.printStackTrace();
		}
	}

}
